-------------------------------------------------------------------------------
-- MrCurrency
--
-- Data Broker source for Currency
--
-- Send suggestions, comments, and bugs to user@zedonline.net
-------------------------------------------------------------------------------

local LibQTip = LibStub("LibQTip-1.0");
MrCurrency = LibStub("AceAddon-3.0"):NewAddon("MrCurrency",
											  "AceBucket-3.0",
											  "AceEvent-3.0",
											  "AceHook-3.0");

local LIGHT_COLOR = {0.4, 0.4, 0.4, 1.0}
local DARK_COLOR = {0, 0, 0.25, 1.0}

-- Used for clicking the Archaeology currency data source
local ArchaeologySpellId = 78670;	
										
function MrCurrency:OnInitialize()
	-- Create our Data Broker Pieces
	local LDB = LibStub("LibDataBroker-1.1", true);
	if LDB then
		-- Money + Currency Display
		self.dataSource = LDB:NewDataObject("MrCurrency", {
	    	type = "data source",
			text = "Loading...",
			label = "MrCurrency",
			icon = "Interface\\Icons\\INV_Misc_Coin_02",
	    	OnClick = function(frame, button)
	        	-- Do something interesting here
				ToggleCharacter("TokenFrame");
	    	end,
			OnEnter = function(frame)
				-- Show tooltip
				MrCurrency:ShowCurrencyTooltip(frame);
			end,
			OnLeave = function()
				-- Clean up tooltip
			    LibQTip:Release(MrCurrency.activeTooltip);
				MrCurrency.activeTooltip = nil;
		    end,
		});
		
		-- Archaeology Currency Display
		local _, _, archIcon = GetSpellInfo(ArchaeologySpellId);
		self.archaeologySource = LDB:NewDataObject("MrCurrency Artifacts", {
			type = "data source",
			text = "Loading...",
			label = "MrCurrency Artifacts",
			icon = archIcon,
			OnClick = function(frame, button)
				-- Show Archaeology Frame
				local spellName = GetSpellInfo(ArchaeologySpellId);
				if ( spellName ) then CastSpellByName(spellName) end;
			end,
			OnEnter = function(frame)
				-- Show tooltip
				MrCurrency:ShowArtifactTooltip(frame);
			end,
			OnLeave = function()
				-- Clean up tooltip
			    LibQTip:Release(MrCurrency.activeTooltip);
				MrCurrency.activeTooltip = nil;
		    end,
		})
	end
	
	self.Hooks:OnEnable();
end

function MrCurrency:OnEnable()
	-- Load our DB
	self.Data:Load();
	
	self:RegisterBucketEvent("WORLD_MAP_UPDATE", 2, "OnMapUpdate"); -- Temporary until Archaeology data is available.
	self:RegisterEvent("PLAYER_MONEY", "OnUpdate");
	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE", "OnUpdate"); -- Also handled by the TokenFrame_Update hook?
	self:RegisterEvent("KNOWN_CURRENCY_TYPES_UPDATE", "OnUpdate");
	self:SecureHook("TokenFrame_LoadUI");
	
	self:OnUpdate();
end

function MrCurrency:TokenFrame_LoadUI()
	self:Unhook("TokenFrame_LoadUI");
	self:SecureHook("TokenFrame_Update", "OnUpdate");	
end

function MrCurrency:OnMapUpdate()
	if( GetNumArchaeologyRaces() > 0 ) then
		self:UpdateArtifacts();
		self:UnregisterEvent("WORLD_MAP_UPDATE");
	end
end

function MrCurrency:OnUpdate()
	self:UpdateCurrency();
	self:UpdateArtifacts();
end

function MrCurrency:UpdateCurrency()
	local money = GetMoney();
	local text = "";

	local currencies = self.Utils:ReadCurrency();
	
	for _, currency in ipairs(currencies) do
		if( not currency.isUnused and currency.count > 0 ) then
			text = self.Utils:AppendCurrency(text, currency.count, currency.icon, nil, 20);
		end
	end
	
	self.Data:StoreCurrencyInfo(money, currencies);
	
	text = self.Utils:AppendMoneyString(text, money, 12);
	
	self.dataSource.text = text;
end

function MrCurrency:ShowCurrencyTooltip(dataFrame)
	local knownCurrencies = self.Data:GetKnownCurrencyList();
	local currencyTotals = {};
	
	local maxColumns = #knownCurrencies + 2;
	local defaultAlignments = {};
	local line = {};
	local lineCount = 1;
	
	wipe(line);
	line[1] = "";
	defaultAlignments[1] = "LEFT"
	
	for _, currency in ipairs(knownCurrencies) do
		line[#line+1] = "|T" .. currency.icon .. ":24:24|t";
		defaultAlignments[#defaultAlignments+1] = "RIGHT";
	end
	
	line[#line+1] = "|TInterface\\Icons\\INV_Misc_Coin_02:24:24|t";
	defaultAlignments[#defaultAlignments+1] = "RIGHT";
	
	self.activeTooltip = LibQTip:Acquire("MrCurrencyTooltip", maxColumns, unpack(defaultAlignments));
	self.activeTooltip:SetCellMarginH(4);
	self.activeTooltip:SetCellMarginV(2);
	self.activeTooltip:AddHeader(unpack(line));
	self.activeTooltip:SetLineColor(lineCount, unpack(DARK_COLOR));
	
	for playerName, playerEntry in self.Utils:OrderedPairs(self.Data:GetCurrencyInfo()) do
		local playerLine = {};
		playerLine[1] = playerName;
		
		for _, currency in ipairs(knownCurrencies) do
			playerLine[#playerLine+1] = playerEntry[currency.icon] or 0;
			currencyTotals[currency.icon] = (currencyTotals[currency.icon] or 0) + (playerEntry[currency.icon] or 0);
		end
		
		playerLine[#playerLine+1] = self.Utils:AppendMoneyString(nil, playerEntry["_money"], 12, 2, true);
		currencyTotals["_money"] = (currencyTotals["_money"] or 0) + (playerEntry["_money"] or 0);
		self.activeTooltip:AddLine(unpack(playerLine));
		lineCount = lineCount + 1;
		
		if( lineCount % 2 == 1 ) then
			self.activeTooltip:SetLineColor(lineCount, unpack(DARK_COLOR));
		else
			self.activeTooltip:SetLineColor(lineCount, unpack(LIGHT_COLOR));
		end
	end
	
	self.activeTooltip:AddSeparator(1.0, 1.0, 1.0, 1.0);
	
	wipe(line);
	line[1] = "Total";
	for _, currency in ipairs(knownCurrencies) do
		line[#line+1] = currencyTotals[currency.icon] or 0;
	end
	line[#line+1] = self.Utils:AppendMoneyString(nil, currencyTotals["_money"] or 0, 12, 2, true);
	self.activeTooltip:AddLine(unpack(line));
	lineCount = lineCount + 1;
	
	if( lineCount % 2 == 1 ) then
		self.activeTooltip:SetLineColor(lineCount + 1, unpack(DARK_COLOR));
	else
		self.activeTooltip:SetLineColor(lineCount + 1, unpack(LIGHT_COLOR));
	end
	
	self.activeTooltip:SetBackdropColor(unpack(DARK_COLOR))
	self.activeTooltip:SmartAnchorTo(dataFrame);
	self.activeTooltip:Show();
end

function MrCurrency:UpdateArtifacts()
	local displayEmpty = true;
	local text = "";
	
	local raceCount = GetNumArchaeologyRaces();
	if( raceCount == 0 ) then return end; -- No Races Means no Archaeology Data
	for index=1,raceCount do
		local _, currencyId, raceIcon, _ = GetArchaeologyRaceInfo(index);
		local currencyName, count, currencyIcon, _, _, _ = GetCurrencyInfo(currencyId);
		
		if( count and count > 0 and self.Data:GetArchaeologyTracking(currencyId)) then
			text = self.Utils:AppendCurrency(text, count, "Interface\\Icons\\" .. currencyIcon, nil, 20);
			displayEmpty = false;
		end
	end
	
	if( displayEmpty ) then text = "No Artifacts" end;
	self.archaeologySource.text = text;
end

function MrCurrency:ShowArtifactTooltip(dataFrame)
	self.activeTooltip = LibQTip:Acquire("MrCurrency_ArtifactsTooltip", 2, "LEFT", "RIGHT");
	local _, _, archIndex = GetProfessions();
	
	if(archIndex == nil) then
		self.activeTooltip:AddLine("Visit a trainer to learn archaeology.");
	else
		local lineCount = 1;
		
		self.activeTooltip:SetCellMarginH(8);
		self.activeTooltip:SetCellMarginV(2);
		self.activeTooltip:AddHeader("Race", "Tokens");
		self.activeTooltip:SetLineColor(lineCount, unpack(DARK_COLOR));

		local raceCount = GetNumArchaeologyRaces();
		for index=1,raceCount do
			local raceName, currencyId, raceIcon, _ = GetArchaeologyRaceInfo(index);
			local currencyName, count, currencyIcon, _, _, _ = GetCurrencyInfo(currencyId);
		
			if( count and count > 0 ) then
				lineCount = lineCount + 1;
				self.activeTooltip:AddLine("|TInterface\\Icons\\" .. currencyIcon .. ":24:24:0:0:0:0:12:12:12:12|t " .. raceName, count .. " ");
			end
			
			if( lineCount % 2 == 1 ) then
				self.activeTooltip:SetLineColor(lineCount, unpack(DARK_COLOR));
			else
				self.activeTooltip:SetLineColor(lineCount, unpack(LIGHT_COLOR));
			end
		end
	end
	
	self.activeTooltip:SmartAnchorTo(dataFrame);
	self.activeTooltip:Show();
end

function MrCurrency:InitializeArchaeologyTrackingList()
	UIDropDownMenu_SetText(MRCArtifactTracking, "Tracking");

	local info = UIDropDownMenu_CreateInfo();
	local raceCount = GetNumArchaeologyRaces();
	for index=1,raceCount do
		local raceName, currencyId, raceIcon, _ = GetArchaeologyRaceInfo(index);
		info.text = raceName;
		info.icon = raceIcon;
		info.value = currencyId;
		info.isNotRadio = true;
		info.checked = self.Data:GetArchaeologyTracking(currencyId);
		info.func = function(this, arg1, arg2, checked) 
			MrCurrency.Data:SetArchaeologyTracking(this.value, not checked);
			MrCurrency:OnUpdate();
		end;
		info.keepDownOnClick = true;
	
		UIDropDownMenu_AddButton(info);
	end
end